@@ -30,6 +30,7 @@
 
 #include <QFile>
 #include <QList>
+#include <QAbstractButton>
 #include <QMessageBox>
 
 #include <kactioncollection.h>
@@ -42,9 +43,9 @@
 #include <klocalizedstring.h>
 #include <kservice.h>
 #include <kservicetypetrader.h>
+#include <KConfigLoader>
 #include <kwindowsystem.h>
 
-#include "configloader.h"
 #include "containment.h"
 #include "corona.h"
 #include "package.h"
@@ -54,7 +55,7 @@
 
 #include "private/associatedapplicationmanager_p.h"
 #include "private/containment_p.h"
-
+#include "private/package_p.h"
 
 namespace Plasma
 {
@@ -90,7 +91,7 @@ Applet::Applet(QObject *parentObject, const QVariantList &args)
 
     // WARNING: do not access config() OR globalConfig() in this method!
     //          that requires a scene, which is not available at this point
-    d->init();
+    d->init(QString(), args.mid(2));
     d->setupPackage();
 }
 
@@ -104,6 +105,9 @@ Applet::Applet(const QString &packagePath, uint appletId)
 
 Applet::~Applet()
 {
+    if (d->transient) {
+        d->resetConfigurationObject();
+    }
     //let people know that i will die
     emit appletDeleted(this);
 
@@ -152,7 +156,7 @@ void Applet::save(KConfigGroup &g) const
         // we're saving so we know its changed, we don't need or want the configChanged
         // signal bubbling up at this point due to that
         disconnect(d->configLoader, SIGNAL(configChanged()), this, SLOT(propagateConfigChanged()));
-        d->configLoader->writeConfig();
+        d->configLoader->save();
         connect(d->configLoader, SIGNAL(configChanged()), this, SLOT(propagateConfigChanged()));
     }
 }
@@ -167,12 +171,12 @@ void Applet::restore(KConfigGroup &group)
     if (!shortcutText.isEmpty()) {
         setGlobalShortcut(QKeySequence(shortcutText));
         /*
-#ifndef NDEBUG
+        #ifndef NDEBUG
         // qDebug() << "got global shortcut for" << name() << "of" << QKeySequence(shortcutText);
-#endif
-#ifndef NDEBUG
+        #endif
+        #ifndef NDEBUG
         // qDebug() << "set to" << d->activationAction->objectName()
-#endif
+        #endif
                  << d->activationAction->globalShortcut().primary();
                  */
     }
@@ -180,10 +184,10 @@ void Applet::restore(KConfigGroup &group)
     // local shortcut, if any
     //TODO: implement; the shortcut will need to be registered with the containment
     /*
-#include "accessmanager.h"
-#include "private/plasmoidservice_p.h"
-#include "authorizationmanager.h"
-#include "authorizationmanager.h"
+    #include "accessmanager.h"
+    #include "private/plasmoidservice_p.h"
+    #include "authorizationmanager.h"
+    #include "authorizationmanager.h"
     shortcutText = shortcutConfig.readEntryUntranslated("local", QString());
     if (!shortcutText.isEmpty()) {
         //TODO: implement; the shortcut
@@ -254,13 +258,8 @@ void Applet::destroy()
         return; //don't double delete
     }
 
-    if (isContainment()) {
-        if (QMessageBox::warning(0, i18nc("@title:window %1 is the name of the containment", "Remove %1", title()), i18nc("%1 is the name of the containment", "Do you really want to remove this %1?", title()), QMessageBox::StandardButtons( QMessageBox::Yes | QMessageBox::No ), QMessageBox::No) != QMessageBox::Yes) {
-            return;
-        }
-    }
-
     d->transient = true;
+    emit destroyedChanged(true);
     //FIXME: an animation on leave if !isContainment() would be good again .. which should be handled by the containment class
     d->cleanUpAndDelete();
 }
@@ -270,16 +269,16 @@ bool Applet::destroyed() const
     return d->transient;
 }
 
-ConfigLoader *Applet::configScheme() const
+KConfigLoader *Applet::configScheme() const
 {
     if (!d->configLoader) {
-        const QString xmlPath = d->package? d->package->filePath("mainconfigxml") : QString();
+        const QString xmlPath = d->package ? d->package->filePath("mainconfigxml") : QString();
         KConfigGroup cfg = config();
         if (xmlPath.isEmpty()) {
-            d->configLoader = new ConfigLoader(&cfg, 0);
+            d->configLoader = new KConfigLoader(cfg, 0);
         } else {
             QFile file(xmlPath);
-            d->configLoader = new ConfigLoader(&cfg, &file);
+            d->configLoader = new KConfigLoader(cfg, &file);
             QObject::connect(d->configLoader, SIGNAL(configChanged()), this, SLOT(propagateConfigChanged()));
         }
     }
@@ -289,7 +288,16 @@ ConfigLoader *Applet::configScheme() const
 
 Package Applet::package() const
 {
-    return d->package ? *d->package : Package();
+    Package p;
+    if (d->package) {
+        p.d->internalPackage = new KPackage::Package(*d->package);
+    }
+    return p;
+}
+
+KPackage::Package Applet::kPackage() const
+{
+    return d->package ? *d->package : KPackage::Package();
 }
 
 void Applet::updateConstraints(Plasma::Types::Constraints constraints)
@@ -344,7 +352,7 @@ KPluginInfo Applet::pluginInfo() const
     return d->appletDescription;
 }
 
-Types::Types::ImmutabilityType Applet::immutability() const
+Types::ImmutabilityType Applet::immutability() const
 {
     // if this object is itself system immutable, then just return that; it's the most
     // restrictive setting possible and will override anything that might be happening above it
@@ -357,7 +365,7 @@ Types::Types::ImmutabilityType Applet::immutability() const
     Types::ImmutabilityType upperImmutability = Types::Mutable;
 
     if (isContainment()) {
-        Corona *cor = static_cast<Containment*>(const_cast<Applet*>(this))->corona();
+        Corona *cor = static_cast<Containment *>(const_cast<Applet *>(this))->corona();
         if (cor) {
             upperImmutability = cor->immutability();
         }
@@ -381,7 +389,7 @@ Types::Types::ImmutabilityType Applet::immutability() const
     }
 }
 
-void Applet::setImmutability(const Types::Types::ImmutabilityType immutable)
+void Applet::setImmutability(const Types::ImmutabilityType immutable)
 {
     if (d->immutability == immutable || immutable == Types::SystemImmutable) {
         // we do not store system immutability in d->immutability since that gets saved
@@ -471,7 +479,7 @@ void Applet::flushPendingConstraintsEvents()
         if (closeApplet) {
             closeApplet->setEnabled(unlocked);
             closeApplet->setVisible(unlocked);
-            connect(closeApplet, SIGNAL(triggered(bool)), this, SLOT(destroy()), Qt::UniqueConnection);
+            connect(closeApplet, SIGNAL(triggered(bool)), this, SLOT(askDestroy()), Qt::UniqueConnection);
         }
 
         QAction *configAction = d->actions->action("configure");
@@ -490,7 +498,6 @@ void Applet::flushPendingConstraintsEvents()
         }
 
         d->updateShortcuts();
-        Containment *cont = containment();
     }
 
     if (c & Plasma::Types::ImmutableConstraint) {
@@ -501,6 +508,12 @@ void Applet::flushPendingConstraintsEvents()
             action->setEnabled(unlocked);
         }
 
+        action = d->actions->action("alternatives");
+        if (action) {
+            action->setVisible(unlocked);
+            action->setEnabled(unlocked);
+        }
+
         action = d->actions->action("configure");
         if (action && d->hasConfigurationInterface) {
             bool canConfig = unlocked || KAuthorized::authorize("plasma/allow_configure_when_locked");
@@ -508,11 +521,16 @@ void Applet::flushPendingConstraintsEvents()
             action->setEnabled(canConfig);
         }
 
-        emit immutabilityChanged(immutability());
+        //an immutable constraint will alwasy happen at startup
+        //make sure don't emit a change signal for nothing
+        if (d->oldImmutability != immutability()) {
+            emit immutabilityChanged(immutability());
+        }
+        d->oldImmutability = immutability();
     }
 
     // now take care of constraints in special subclass: Contaiment
-    Containment* containment = qobject_cast<Plasma::Containment*>(this);
+    Containment *containment = qobject_cast<Plasma::Containment *>(this);
     if (containment) {
         containment->d->containmentConstraintsEvent(c);
     }
@@ -530,12 +548,20 @@ void Applet::flushPendingConstraintsEvents()
             d->modificationsTimer = new QBasicTimer;
         }
     }
+
+    if (c & Plasma::Types::FormFactorConstraint) {
+        emit formFactorChanged(formFactor());
+    }
+
+    if (c & Plasma::Types::LocationConstraint) {
+        emit locationChanged(location());
+    }
 }
 
-QList<QAction*> Applet::contextualActions()
+QList<QAction *> Applet::contextualActions()
 {
     //qDebug() << "empty context actions";
-    return d->script ? d->script->contextualActions() : QList<QAction*>();
+    return d->script ? d->script->contextualActions() : QList<QAction *>();
 }
 
 KActionCollection *Applet::actions() const
@@ -559,36 +585,25 @@ Types::FormFactor Applet::formFactor() const
 
 Containment *Applet::containment() const
 {
-    Containment *c = qobject_cast<Containment*>(const_cast<Applet*>(this));
-    if (c) {
+    Containment *c = qobject_cast<Containment *>(const_cast<Applet *>(this));
+    if (c && c->isContainment()) {
         return c;
+    } else {
+        c = 0;
     }
 
     QObject *parent = this->parent();
 
     while (parent) {
-        Containment *possibleC = qobject_cast<Containment*>(parent);
-        if (possibleC) {
+        Containment *possibleC = qobject_cast<Containment *>(parent);
+
+        if (possibleC && possibleC->isContainment()) {
             c = possibleC;
             break;
         }
         parent = parent->parent();
     }
 
-    if (!c) {
-        //if the applet is an offscreen widget its parentItem will be 0, while its parent
-        //will be its parentWidget, so here we check the QObject hierarchy.
-        QObject *objParent = this->parent();
-        while (objParent) {
-            Containment *possibleC = qobject_cast<Containment*>(objParent);
-            if (possibleC) {
-                c = possibleC;
-                break;
-            }
-            objParent = objParent->parent();
-        }
-    }
-
     return c;
 }
 
@@ -599,23 +614,27 @@ void Applet::setGlobalShortcut(const QKeySequence &shortcut)
         d->activationAction->setText(i18n("Activate %1 Widget", title()));
         d->activationAction->setObjectName(QString("activate widget %1").arg(id())); // NO I18N
         connect(d->activationAction, SIGNAL(triggered()), this, SIGNAL(activated()));
-        connect(d->activationAction, SIGNAL(globalShortcutChanged(QKeySequence)),
+        connect(d->activationAction, SIGNAL(changed()),
                 this, SLOT(globalShortcutChanged()));
     } else if (d->activationAction->shortcut() == shortcut) {
         return;
     }
 
+    d->activationAction->setShortcut(shortcut);
     d->globalShortcutEnabled = true;
     QList<QKeySequence> seqs;
     seqs << shortcut;
-    KGlobalAccel::self()->setDefaultShortcut(d->activationAction, seqs, KGlobalAccel::NoAutoloading);
+    KGlobalAccel::self()->setShortcut(d->activationAction, seqs, KGlobalAccel::NoAutoloading);
     d->globalShortcutChanged();
 }
 
 QKeySequence Applet::globalShortcut() const
 {
     if (d->activationAction) {
-        return d->activationAction->shortcut();
+        QList<QKeySequence> shortcuts = KGlobalAccel::self()->shortcut(d->activationAction);
+        if (!shortcuts.isEmpty()) {
+            return shortcuts.first();
+        }
     }
 
     return QKeySequence();
@@ -655,7 +674,7 @@ void Applet::configChanged()
 {
     if (d->script) {
         if (d->configLoader) {
-            d->configLoader->readConfig();
+            d->configLoader->load();
         }
         d->script->configChanged();
     }
@@ -736,7 +755,7 @@ void Applet::timerEvent(QTimerEvent *event)
 
         // Don't flushPendingConstraints if we're just starting up
         // flushPendingConstraints will be called by Corona
-        if(!(d->pendingConstraints & Plasma::Types::StartupCompletedConstraint)) {
+        if (!(d->pendingConstraints & Plasma::Types::StartupCompletedConstraint)) {
             flushPendingConstraintsEvents();
         }
     } else if (d->modificationsTimer && event->timerId() == d->modificationsTimer->timerId()) {
@@ -751,7 +770,14 @@ void Applet::timerEvent(QTimerEvent *event)
 
 bool Applet::isContainment() const
 {
-    return qobject_cast<const Containment*>(this);
+    //HACK: this is a special case for the systray
+    //containment in an applet that is not a containment
+    Applet *pa = qobject_cast<Applet *>(parent());
+    if (pa && !pa->isContainment()) {
+        return true;
+    }
+    //normal "acting as a containment" condition
+    return qobject_cast<const Containment *>(this) && qobject_cast<Corona *>(parent());
 }
 
 } // Plasma namespace
