@@ -24,6 +24,7 @@
 #include <kservice.h>
 #include <kservicetypetrader.h>
 #include <kshell.h>
+#include <kconfig.h>
 #include <ksycoca.h>
 #include <klocalizedstring.h>
 
@@ -44,11 +45,14 @@
 #include <iostream>
 #include <iomanip>
 
+#include "config-plasma.h"
+
 static QTextStream cout(stdout);
 
 namespace Plasma
 {
-class PlasmaPkgPrivate {
+class PlasmaPkgPrivate
+{
 public:
     QString packageRoot;
     QString packageFile;
@@ -61,7 +65,7 @@ public:
     QString installPath;
     void output(const QString &msg);
     void runKbuildsycoca();
-    QStringList packages(const QStringList& types);
+    QStringList packages(const QStringList &types);
     void renderTypeTable(const QMap<QString, QStringList> &plugins);
     void listTypes();
     void coutput(const QString &msg);
@@ -69,7 +73,7 @@ public:
 
 };
 
-PlasmaPkg::PlasmaPkg(int& argc, char** argv, QCommandLineParser *parser) :
+PlasmaPkg::PlasmaPkg(int &argc, char **argv, QCommandLineParser *parser) :
     QCoreApplication(argc, argv)
 {
     d = new PlasmaPkgPrivate;
@@ -84,7 +88,7 @@ PlasmaPkg::~PlasmaPkg()
 
 void PlasmaPkg::runMain()
 {
-    Plasma::PackageStructure* structure = new Plasma::PackageStructure;
+    Plasma::PackageStructure *structure = new Plasma::PackageStructure;
     if (d->parser->isSet("hash")) {
         const QString path = d->parser->value("hash");
         Plasma::Package package(structure);
@@ -92,7 +96,7 @@ void PlasmaPkg::runMain()
         const QString hash = package.contentsHash();
         if (hash.isEmpty()) {
             d->coutput(i18n("Failed to generate a Package hash for %1", path));
-            exit(1);
+            exit(9);
         } else {
             d->coutput(i18n("SHA1 hash for Package at %1: '%2'", package.path(), hash));
             exit(0);
@@ -132,8 +136,8 @@ void PlasmaPkg::runMain()
     }
 
     if (!d->packageFile.isEmpty() && (!d->parser->isSet("type") ||
-        type.compare(i18nc("package type", "wallpaper"), Qt::CaseInsensitive) == 0 ||
-        type.compare("wallpaper", Qt::CaseInsensitive) == 0)) {
+                                      type.compare(i18nc("package type", "wallpaper"), Qt::CaseInsensitive) == 0 ||
+                                      type.compare("wallpaper", Qt::CaseInsensitive) == 0)) {
         // Check type for common plasma packages
         Plasma::Package package(structure);
         QString serviceType;
@@ -142,14 +146,14 @@ void PlasmaPkg::runMain()
         } else {
             package.setPath(d->packageFile);
         }
-        if (package.isValid()) {
+        if (package.isValid() && package.metadata().isValid()) {
             serviceType = package.metadata().property("X-Plasma-ServiceType").toString();
         }
 
         if (!serviceType.isEmpty()) {
             if (serviceType.contains("Plasma/Applet") ||
-                //serviceType.contains("Plasma/PopupApplet") ||
-                serviceType.contains("Plasma/Containment")) {
+                    //serviceType.contains("Plasma/PopupApplet") ||
+                    serviceType.contains("Plasma/Containment")) {
                 type = "plasmoid";
             } else if (serviceType == "Plasma/Generic") {
                 type = "package";
@@ -186,53 +190,53 @@ void PlasmaPkg::runMain()
     }
 
     if (type.compare(i18nc("package type", "plasmoid"), Qt::CaseInsensitive) == 0 ||
-        type.compare("plasmoid", Qt::CaseInsensitive) == 0) {
-        d->packageRoot = "plasma/plasmoids/";
+            type.compare("plasmoid", Qt::CaseInsensitive) == 0) {
+        d->packageRoot = PLASMA_RELATIVE_DATA_INSTALL_DIR "/plasmoids/";
         d->servicePrefix = "plasma-applet-";
         d->pluginTypes << "Plasma/Applet";
         d->pluginTypes << "Plasma/Containment";
-    } else if (type.compare(i18nc("package type", "package"), Qt::CaseInsensitive) == 0 /*||
-               type.compare("theme", Qt::CaseInsensitive) == 0*/) {
-        d->packageRoot = "plasma/packages/";
+    } else if (type.compare(i18nc("package type", "package"), Qt::CaseInsensitive) == 0 ||
+               type.compare("package", Qt::CaseInsensitive) == 0) {
+        d->packageRoot = PLASMA_RELATIVE_DATA_INSTALL_DIR "/packages/";
         d->servicePrefix = "plasma-package-";
         d->pluginTypes << "Plasma/Generic";
     } else if (type.compare(i18nc("package type", "theme"), Qt::CaseInsensitive) == 0 ||
                type.compare("theme", Qt::CaseInsensitive) == 0) {
-        d->packageRoot = "desktoptheme/";
+        d->packageRoot = PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/";
         d->pluginTypes << "Plasma/Theme";
     } else if (type.compare(i18nc("package type", "wallpaper"), Qt::CaseInsensitive) == 0 ||
                type.compare("wallpaper", Qt::CaseInsensitive) == 0) {
-        d->pluginTypes << "Plasma/Wallpaper";
-        d->packageRoot = "plasma/wallpapers/";
+        d->pluginTypes << "Plasma/ImageWallpaper"; // we'll catch that later
+        d->packageRoot = "wallpapers/";
         d->servicePrefix = "plasma-wallpaper-";
     } else if (type.compare(i18nc("package type", "dataengine"), Qt::CaseInsensitive) == 0 ||
                type.compare("dataengine", Qt::CaseInsensitive) == 0) {
-        d->packageRoot = "plasma/dataengines/";
+        d->packageRoot = PLASMA_RELATIVE_DATA_INSTALL_DIR "/dataengines/";
         d->servicePrefix = "plasma-dataengine-";
         d->pluginTypes << "Plasma/DataEngine";
     } else if (type.compare(i18nc("package type", "runner"), Qt::CaseInsensitive) == 0 ||
                type.compare("runner", Qt::CaseInsensitive) == 0) {
-        d->packageRoot = "plasma/runners/";
+        d->packageRoot = PLASMA_RELATIVE_DATA_INSTALL_DIR "/runners/";
         d->servicePrefix = "plasma-runner-";
         d->pluginTypes << "Plasma/Runner";
     } else if (type.compare(i18nc("package type", "wallpaperplugin"), Qt::CaseInsensitive) == 0 ||
                type.compare("wallpaperplugin", Qt::CaseInsensitive) == 0) {
-        d->packageRoot = "plasma/wallpapers/";
+        d->packageRoot = PLASMA_RELATIVE_DATA_INSTALL_DIR "/wallpapers/";
         d->servicePrefix = "plasma-wallpaper-";
         d->pluginTypes << "Plasma/Wallpaper";
     } else if (type.compare(i18nc("package type", "lookandfeel"), Qt::CaseInsensitive) == 0 ||
                type.compare("lookandfeel", Qt::CaseInsensitive) == 0) {
-        d->packageRoot = "plasma/look-and-feel/";
+        d->packageRoot = PLASMA_RELATIVE_DATA_INSTALL_DIR "/look-and-feel/";
         d->servicePrefix = "plasma-lookandfeel-";
         d->pluginTypes << "Plasma/LookAndFeel";
     } else if (type.compare(i18nc("package type", "shell"), Qt::CaseInsensitive) == 0 ||
                type.compare("shell", Qt::CaseInsensitive) == 0) {
-        d->packageRoot = "plasma/shells/";
+        d->packageRoot = PLASMA_RELATIVE_DATA_INSTALL_DIR "/shells/";
         d->servicePrefix = "plasma-shell-";
         d->pluginTypes << "Plasma/Shell";
     } else if (type.compare(i18nc("package type", "layout-template"), Qt::CaseInsensitive) == 0 ||
                type.compare("layout-template", Qt::CaseInsensitive) == 0) {
-        d->packageRoot = "plasma/layout-templates/";
+        d->packageRoot = PLASMA_RELATIVE_DATA_INSTALL_DIR "/layout-templates/";
         d->servicePrefix = "plasma-layout-";
         d->pluginTypes << "Plasma/LayoutTemplate";
     } else if (type.compare(i18nc("package type", "kwineffect"), Qt::CaseInsensitive) == 0 ||
@@ -250,12 +254,12 @@ void PlasmaPkg::runMain()
         d->packageRoot = "kwin/scripts/";
         d->servicePrefix = "kwin-script-";
         d->pluginTypes << "KWin/Script";
-    } else /* if (KSycoca::isAvailable()) */ {
+    } else { /* if (KSycoca::isAvailable()) */
         const QString constraint = QString("[X-KDE-PluginInfo-Name] == '%1'").arg(type);
         KService::List offers = KServiceTypeTrader::self()->query("Plasma/PackageStructure", constraint);
         if (offers.isEmpty()) {
             d->coutput(i18n("Could not find a suitable installer for package of type %1", type));
-            exit(1);
+            exit(5);
             return;
         }
         qWarning() << "custom PackageStructure plugins not ported";
@@ -266,7 +270,7 @@ void PlasmaPkg::runMain()
 
         if (!d->installer) {
             d->coutput(i18n("Could not load installer for package of type %1. Error reported was: %2",
-                        d->parser->value("type"), error));
+                            d->parser->value("type"), error));
             return;
         }
 
@@ -355,7 +359,7 @@ void PlasmaPkg::runMain()
                 return;
             } else {
                 d->coutput(i18n("Error: Plugin %1 is not installed.", pluginName));
-                exit(1);
+                exit(2);
             }
         }
         if (d->parser->isSet("install")) {
@@ -365,7 +369,7 @@ void PlasmaPkg::runMain()
         }
         if (d->package.isEmpty()) {
             qWarning() << i18nc("No option was given, this is the error message telling the user he needs at least one, do not translate install, remove, upgrade nor list", "One of install, remove, upgrade or list is required.");
-            exit(1);
+            exit(6);
         } else {
             d->runKbuildsycoca();
         }
@@ -387,20 +391,35 @@ void PlasmaPkgPrivate::runKbuildsycoca()
     }
 }
 
-QStringList PlasmaPkgPrivate::packages(const QStringList& types)
+QStringList PlasmaPkgPrivate::packages(const QStringList &types)
 {
     QStringList result;
 
-    foreach (const QString& type, types) {
+    foreach (const QString &type, types) {
 
         if (type.compare("Plasma/Generic", Qt::CaseInsensitive) == 0) {
-            const QStringList &packs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, "plasma/packages/", QStandardPaths::LocateDirectory);
+            const QStringList &packs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, PLASMA_RELATIVE_DATA_INSTALL_DIR "/packages/", QStandardPaths::LocateDirectory);
             foreach (const QString &ppath, packs) {
                 const QDir cd(ppath);
                 const QStringList &entries = cd.entryList(QDir::Dirs);
                 foreach (const QString pack, entries) {
                     if ((pack != "." && pack != "..") &&
-                        (QFile::exists(ppath+'/'+pack+"/metadata.desktop"))) {
+                            (QFile::exists(ppath + '/' + pack + "/metadata.desktop"))) {
+
+                        result << pack;
+                    }
+                }
+            }
+        }
+
+        if (type.compare("Plasma/ImageWallpaper", Qt::CaseInsensitive) == 0) {
+            const QStringList &packs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, "wallpapers/", QStandardPaths::LocateDirectory);
+            foreach (const QString &ppath, packs) {
+                const QDir cd(ppath);
+                const QStringList &entries = cd.entryList(QDir::Dirs);
+                foreach (const QString pack, entries) {
+                    if ((pack != "." && pack != "..") &&
+                            (QFile::exists(ppath + '/' + pack + "/metadata.desktop"))) {
 
                         result << pack;
                     }
@@ -409,13 +428,13 @@ QStringList PlasmaPkgPrivate::packages(const QStringList& types)
         }
 
         if (type.compare("Plasma/Theme", Qt::CaseInsensitive) == 0) {
-            const QStringList &packs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, "desktoptheme/", QStandardPaths::LocateDirectory);
+            const QStringList &packs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/", QStandardPaths::LocateDirectory);
             foreach (const QString &ppath, packs) {
                 const QDir cd(ppath);
                 const QStringList &entries = cd.entryList(QDir::Dirs);
                 foreach (const QString pack, entries) {
                     if ((pack != "." && pack != "..") &&
-                        (QFile::exists(ppath+'/'+pack+"/metadata.desktop"))) {
+                            (QFile::exists(ppath + '/' + pack + "/metadata.desktop"))) {
 
                         result << pack;
                     }
@@ -435,7 +454,7 @@ QStringList PlasmaPkgPrivate::packages(const QStringList& types)
     return result;
 }
 
-void PlasmaPkg::showPackageInfo(const QString& pluginName)
+void PlasmaPkg::showPackageInfo(const QString &pluginName)
 {
     QString type = QStringLiteral("Plasma/Applet");
     if (!d->pluginTypes.contains(type) && d->pluginTypes.count() > 0) {
@@ -454,7 +473,7 @@ void PlasmaPkg::showPackageInfo(const QString& pluginName)
     KPluginInfo i = pkg.metadata();
     if (!i.isValid()) {
         d->coutput(i18n("Error: Can't find plugin metadata: %1", pluginName));
-        exit(1);
+        exit(3);
     }
     d->coutput(i18n("Showing info for package: %1", pluginName));
     d->coutput(i18n("      Name : %1", i.name()));
@@ -466,12 +485,12 @@ void PlasmaPkg::showPackageInfo(const QString& pluginName)
     exit(0);
 }
 
-QString PlasmaPkg::findPackageRoot(const QString& pluginName, const QString& prefix)
+QString PlasmaPkg::findPackageRoot(const QString &pluginName, const QString &prefix)
 {
     QString packageRoot;
     if (d->parser->isSet("packageroot") && d->parser->isSet("global")) {
         qWarning() << i18nc("The user entered conflicting options packageroot and global, this is the error message telling the user he can use only one", "The packageroot and global options conflict each other, please select only one.");
-        ::exit(1);
+        ::exit(7);
     } else if (d->parser->isSet("packageroot")) {
         packageRoot = d->parser->value("packageroot");
         //qDebug() << "(set via arg) d->packageRoot is: " << d->packageRoot;
@@ -483,11 +502,11 @@ QString PlasmaPkg::findPackageRoot(const QString& pluginName, const QString& pre
     return packageRoot;
 }
 
-void PlasmaPkg::listPackages(const QStringList& types)
+void PlasmaPkg::listPackages(const QStringList &types)
 {
     QStringList list = d->packages(types);
     list.sort();
-    foreach (const QString& package, list) {
+    foreach (const QString &package, list) {
         d->coutput(package);
     }
     exit(0);
@@ -550,16 +569,16 @@ void PlasmaPkgPrivate::listTypes()
     coutput(i18n("Built in:"));
 
     QMap<QString, QStringList> builtIns;
-    builtIns.insert(i18n("DataEngine"), QStringList() << "Plasma/DataEngine" << "plasma/dataengines/" << "dataengine");
-    builtIns.insert(i18n("Layout Template"), QStringList() << "Plasma/LayoutTemplate" << "plasma/layout-templates/" << "layout-template");
-    builtIns.insert(i18n("Look and Feel"), QStringList() << "Plasma/LookAndFeel" << "plasma/look-and-feel/" << "lookandfeel");
-    builtIns.insert(i18n("Package"), QStringList() << "Plasma/Generic" << "plasma/packages/" << "package");
-    builtIns.insert(i18n("Plasmoid"), QStringList() << "Plasma/Applet" << "plasma/plasmoids/" << "plasmoid");
-    builtIns.insert(i18n("Runner"), QStringList() << "Plasma/Runner" << "plasma/runners/" << "runner");
-    builtIns.insert(i18n("Shell"), QStringList() << "Plasma/Shell" << "plasma/shells/" << "shell");
-    builtIns.insert(i18n("Theme"), QStringList() << "" << "desktoptheme/" << "theme");
+    builtIns.insert(i18n("DataEngine"), QStringList() << "Plasma/DataEngine" << PLASMA_RELATIVE_DATA_INSTALL_DIR "/dataengines/" << "dataengine");
+    builtIns.insert(i18n("Layout Template"), QStringList() << "Plasma/LayoutTemplate" << PLASMA_RELATIVE_DATA_INSTALL_DIR "/layout-templates/" << "layout-template");
+    builtIns.insert(i18n("Look and Feel"), QStringList() << "Plasma/LookAndFeel" << PLASMA_RELATIVE_DATA_INSTALL_DIR "/look-and-feel/" << "lookandfeel");
+    builtIns.insert(i18n("Package"), QStringList() << "Plasma/Generic" << PLASMA_RELATIVE_DATA_INSTALL_DIR "/packages/" << "package");
+    builtIns.insert(i18n("Plasmoid"), QStringList() << "Plasma/Applet" << PLASMA_RELATIVE_DATA_INSTALL_DIR "/plasmoids/" << "plasmoid");
+    builtIns.insert(i18n("Runner"), QStringList() << "Plasma/Runner" << PLASMA_RELATIVE_DATA_INSTALL_DIR "/runners/" << "runner");
+    builtIns.insert(i18n("Shell"), QStringList() << "Plasma/Shell" << PLASMA_RELATIVE_DATA_INSTALL_DIR "/shells/" << "shell");
+    builtIns.insert(i18n("Theme"), QStringList() << "" << PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/" << "theme");
     builtIns.insert(i18n("Wallpaper Images"), QStringList() << "" << "wallpapers/" << "wallpaper");
-    builtIns.insert(i18n("Wallpaper Plugin"), QStringList() << "Plasma/Wallpaper" << "plasma/wallpapers/" << "wallpaperplugin");
+    builtIns.insert(i18n("Animated Wallpaper"), QStringList() << "Plasma/Wallpaper" << PLASMA_RELATIVE_DATA_INSTALL_DIR "/wallpapers/" << "wallpaperplugin");
     builtIns.insert(i18n("KWin Effect"), QStringList() << "KWin/Effect" << "kwin/effects/" << "kwineffect");
     builtIns.insert(i18n("KWin Window Switcher"), QStringList() << "KWin/WindowSwitcher" << "kwin/tabbox/" << "windowswitcher");
     builtIns.insert(i18n("KWin Script"), QStringList() << "KWin/Script" << "kwin/scripts/" << "kwinscript");
@@ -590,7 +609,7 @@ void PlasmaPkgPrivate::listTypes()
         renderTypeTable(plugins);
     }
 
-    QStringList desktopFiles = QStandardPaths::locateAll(QStandardPaths::DataLocation, "plasma/packageformats/*rc", QStandardPaths::LocateFile);
+    QStringList desktopFiles = QStandardPaths::locateAll(QStandardPaths::DataLocation, PLASMA_RELATIVE_DATA_INSTALL_DIR "/packageformats/*rc", QStandardPaths::LocateFile);
 
     if (!desktopFiles.isEmpty()) {
         coutput(i18n("Provided by .desktop files:"));
@@ -599,7 +618,7 @@ void PlasmaPkgPrivate::listTypes()
         foreach (const QString &file, desktopFiles) {
             // extract the type
             KConfig config(file, KConfig::SimpleConfig);
-            #warning "read config here"
+#pragma message("read config here")
             // structure.read(&config);
             // get the name based on the rc file name, just as Plasma::PackageStructure does
             const QString name = file.left(file.length() - 2);
@@ -620,7 +639,7 @@ void PlasmaPkg::packageInstalled(KJob *job)
         }
     } else {
         d->coutput(i18n("Error: Installation of %1 failed: %2", d->packageFile, job->errorText()));
-        exitcode = 1;
+        exitcode = 4;
     }
     exit(exitcode);
 }
@@ -639,7 +658,7 @@ void PlasmaPkg::packageUninstalled(KJob *job)
         d->coutput(i18n("Successfully uninstalled %1", d->packageFile));
     } else {
         d->coutput(i18n("Error: Uninstallation of %1 failed: %2", d->packageFile, job->errorText()));
-        exitcode = 1;
+        exitcode = 7;
     }
     exit(exitcode);
 }
